% MATLAB demo code of Reweighted Random Walks Hyper-graph Matching of CVPR 2011
% 
% Jungmin Lee, Minsu Cho, and Kyoung Mu Lee, 
% Hyper-graph Matching via Reweighted Random Walks
% Proc. Computer Vision and Pattern Recognition (CVPR), 2011
% http://cv.snu.ac.kr/research/~RRWHM/
% 
% written by Jungmin Lee & Minsu Cho, Seoul National University, Korea
% http://cv.snu.ac.kr/~jungminlee/
% http://cv.snu.ac.kr/~minsucho/

%% Test hyper-graph matching with various approaches
% Jungmin Lee in Computer Vision Lab
clear all; close all; clc;
disp('************************ Welcom to the Hyper-graph Matching world!! ************************');disp(' ');

%% Settings Evaluations
setPath; setSettings; setAlg;
disp('* Check experiment settings *'); disp(Set);

%% Test Methods
Accuracy = zeros(length(settings{Con}{4}), length(Alg), Set.nTest);
MatchScore = zeros(length(settings{Con}{4}), length(Alg), Set.nTest);
Time = zeros(length(settings{Con}{4}), length(Alg), Set.nTest);

for k = 1:Set.nTest, fprintf('Test: %d of %d ', k, Set.nTest);
    for i = 1:length(settings{Con}{4})
        eval(['Set.' settings{Con}{3} '=' num2str(settings{Con}{4}(i)) ';']);
        problem = makeMatchingProblem(Set);
        eval(['Set.' settings{Con}{3} '= settings{' num2str(Con) '}{4};']);
        for j = 1:length(Alg)
            tic;
            X = feval(Alg(j).fhandle, Alg(j), problem);
            Time(i,j,k) = toc;
            X = makeSol1to1(X);
            [val ind] = max(X);
            Xbin = zeros(size(X,1), size(X,2));
            for p = 1:size(X,2), Xbin(ind(p),p) = 1; end
            Accuracy(i,j,k) = (sum(ind == problem.trueMatch))/problem.nP1;
            MatchScore(i,j,k) = getMatchScore(problem.indH3, problem.valH3, Xbin);
        end
        fprintf('.');
    end
    fprintf('\n');
end
clear i j k temp X Xbin ind p val

%% Plot Results
meanAccuracy = mean(Accuracy,3);
meanMatchScore = mean(MatchScore,3);
meanTime = mean(Time,3);

plotSet.lineWidth = 3; % Line width
plotSet.markerSize = 10; % Marker Size
plotSet.fontSize = 20; % Font Size
plotSet.font = '\fontname{times new roman}'; % Font default

handleCount = 0;
yData = meanAccuracy; yLabelText = 'Accuracy'; plotResults;
yData = meanMatchScore; yLabelText = 'Matching Score'; plotResults;
yData = meanTime; yLabelText = 'Time'; plotResults;